@@ -2,7 +2,11 @@ use {
 	crate::result::Result,
 	serde::{Deserialize, Serialize},
 	sqlparser::ast::DataType,
-	std::{cmp::Ordering, fmt::Debug},
+	std::{
+		cmp::Ordering,
+		fmt::Debug,
+		hash::{Hash, Hasher},
+	},
 };
 
 mod big_endian;
@@ -66,7 +70,7 @@ pub enum Value {
 	Null,
 
 	Bool(bool),
-	U64(i64),
+	U64(u64),
 	I64(i64),
 	F64(f64),
 	Str(String),
@@ -77,6 +81,18 @@ pub enum Value {
 	Internal(i64),
 }
 
+impl Hash for Value {
+	fn hash<H: Hasher>(&self, state: &mut H) {
+		self.to_be_bytes().hash(state)
+	}
+}
+impl Eq for Value {}
+impl Ord for Value {
+	fn cmp(&self, other: &Self) -> Ordering {
+		self.partial_cmp(other).unwrap_or(Ordering::Equal)
+	}
+}
+
 impl From<bool> for Value {
 	fn from(from: bool) -> Value {
 		Value::Bool(from)
